@@ -77,8 +77,7 @@ GST_DEBUG_CATEGORY (cheese_camera_device_cat);
 #define GST_CAT_DEFAULT cheese_camera_device_cat
 
 static gchar *supported_formats[] = {
-  "video/x-raw-rgb",
-  "video/x-raw-yuv",
+  "video/x-raw",
   NULL
 };
 
@@ -296,6 +295,14 @@ cheese_camera_device_update_format_table (CheeseCameraDevice *device)
       min_height = gst_value_get_int_range_min (height);
       max_height = gst_value_get_int_range_max (height);
 
+      /* Some devices report a very small min_width / height down to reporting
+       * 0x0 as minimum resolution, which causes an infinte loop below, limit
+       * these to something reasonable. */
+      if (min_width < 160)
+        min_width = 160;
+      if (min_height < 120)
+        min_height = 120;
+
       cur_width  = min_width;
       cur_height = min_height;
 
@@ -305,8 +312,10 @@ cheese_camera_device_update_format_table (CheeseCameraDevice *device)
       {
         CheeseVideoFormat *format = g_slice_new0 (CheeseVideoFormat);
 
-        format->width  = cur_width;
-        format->height = cur_height;
+        /* Gstreamer wants resolutions for YUV formats where the width is
+         * a multiple of 8, and the height is a multiple of 2 */
+        format->width  = cur_width & ~7;
+        format->height = cur_height & ~1;
 
         cheese_camera_device_add_format (device, format);
 
@@ -320,8 +329,10 @@ cheese_camera_device_update_format_table (CheeseCameraDevice *device)
       {
         CheeseVideoFormat *format = g_slice_new0 (CheeseVideoFormat);
 
-        format->width  = cur_width;
-        format->height = cur_height;
+        /* Gstreamer wants resolutions for YUV formats where the width is
+         * a multiple of 8, and the height is a multiple of 2 */
+        format->width  = cur_width & ~7;
+        format->height = cur_height & ~1;
 
         cheese_camera_device_add_format (device, format);
 
@@ -377,8 +388,8 @@ cheese_camera_device_get_caps (CheeseCameraDevice *device)
       src = gst_bin_get_by_name (GST_BIN (pipeline), "source");
 
       GST_LOG ("Device: %s (%s)\n", priv->name, priv->device_node);
-      pad        = gst_element_get_pad (src, "src");
-      caps       = gst_pad_get_caps (pad);
+      pad        = gst_element_get_static_pad (src, "src");
+      caps       = gst_pad_get_allowed_caps (pad);
       priv->caps = cheese_camera_device_filter_caps (device, caps, supported_formats);
       if (!gst_caps_is_empty (priv->caps))
         cheese_camera_device_update_format_table (device);
